@@ -1,7 +1,7 @@
 /*
    anubisusr.c
    
-   Copyright (C) 2004-2020 The Anubis Team.
+   Copyright (C) 2004-2024 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -19,10 +19,8 @@
 
 #include <anubisusr.h>
 
-#ifdef USE_GNUTLS
 struct secure_struct secure;
 int enable_tls = 1;
-#endif /* USE_GNUTLS */
 
 char *progname;
 
@@ -30,7 +28,6 @@ char *smtp_host = "localhost";
 int smtp_port = 24;
 char *rcfile_name = NULL;
 char *netrc_name = NULL;
-struct obstack input_stk;
 
 int verbose;
 
@@ -63,9 +60,6 @@ error (const char *fmt, ...)
 
 NET_STREAM iostream;
 
-
-#ifdef USE_GNUTLS
-
 void
 info (int mode, const char *fmt, ...)
 {
@@ -115,7 +109,6 @@ starttls (void)
       exit (1);
     }
 }
-#endif /* USE_GNUTLS */
 
 
 /* Auxiliary functions */
@@ -212,28 +205,31 @@ find_capa_v (ANUBIS_SMTP_REPLY repl, const char *name, ANUBIS_LIST list)
   if (smtp_reply_has_capa (repl, name, &n))
     {
       const char *str = smtp_reply_line (repl, n);
-      int i, argc;
-      char **argv;
+      size_t i;
+      wordsplit_t ws;
       char *rv = NULL;
-      int rc;
-      
-      if ((rc = argcv_get (str, "", NULL, &argc, &argv)))
+
+      if (wordsplit (str, &ws,
+		     WRDSF_NOVAR | WRDSF_NOCMD | WRDSF_SQUEEZE_DELIMS))
 	{
-	  error (_("argcv_get failed: %s"), strerror (rc));
+	  error (_("wordsplit failed: %s"), wordsplit_strerror (&ws));
+	  wordsplit_free (&ws);
 	  return NULL;
 	}
 
       if (!list)
 	{
-	  if (argv[1])
-	    rv = xstrdup (argv[1]);
+	  if (ws.ws_wordv[1])
+	    rv = ws.ws_wordv[1];
 	}
       else
 	{
-	  for (i = 0; !rv && i < argc; i++)
-	    rv = list_locate (list, argv[i], name_cmp);
+	  for (i = 0; !rv && i < ws.ws_wordc; i++)
+	    rv = list_locate (list, ws.ws_wordv[i], name_cmp);
 	}
-      argcv_free (argc, argv);
+      if (rv)
+	rv = xstrdup (rv);
+      wordsplit_free (&ws);
       return rv;
     }
   return NULL;
@@ -401,10 +397,12 @@ parse_netrc (const char *filename)
   FILE *fp;
   char *buf = NULL;
   size_t n = 0;
-  int def_argc = 0;
+  size_t def_argc = 0;
   char **def_argv = NULL;
   char **p_argv = NULL;
   int line = 0;
+  wordsplit_t ws = { .ws_comment = "#" };
+  int wsflags = WRDSF_NOVAR | WRDSF_NOCMD | WRDSF_SQUEEZE_DELIMS | WRDSF_COMMENT;
 
   fp = fopen (filename, "r");
   if (!fp)
@@ -419,13 +417,11 @@ parse_netrc (const char *filename)
   else
     VDETAIL (1, (_("Opening configuration file %s...\n"), filename));
 
-  while (getline (&buf, &n, fp) > 0 && n > 0)
+  while (xgetline (&buf, &n, fp) > 0 && n > 0)
     {
       int rc;
       char *p;
       size_t len;
-      int argc;
-      char **argv;
 
       line++;
       len = strlen (buf);
@@ -435,45 +431,47 @@ parse_netrc (const char *filename)
       if (*p == 0 || *p == '#')
 	continue;
 
-      if ((rc = argcv_get (buf, "", "#", &argc, &argv)))
+      rc = wordsplit (p, &ws, wsflags);
+      wsflags |= WRDSF_REUSE;
+      if (rc)
 	{
-	  error (_("argcv_get failed: %s"), strerror (rc));
-	  return;
+	  error (_("wordsplit failed: %s"), wordsplit_strerror (&ws));
+	  break;
 	}
       
-      if (strcmp (argv[0], "machine") == 0)
+      if (strcmp (ws.ws_wordv[0], "machine") == 0)
 	{
-	  if (hostcmp (argv[1], smtp_host) == 0)
+	  if (hostcmp (ws.ws_wordv[1], smtp_host) == 0)
 	    {
 	      VDETAIL (1, (_("Found matching line %d\n"), line));
 
 	      if (def_argc)
-		argcv_free (def_argc, def_argv);
-	      def_argc = argc;
-	      def_argv = argv;
-	      p_argv = argv + 2;
+		argv_free (def_argv);
+	      wordsplit_get_words (&ws, &def_argc, &def_argv);
+	      p_argv = def_argv + 2;
 	      break;
 	    }
 	}
-      else if (strcmp (argv[0], "default") == 0)
+      else if (strcmp (ws.ws_wordv[0], "default") == 0)
 	{
 	  VDETAIL (1, (_("Found default line %d\n"), line));
 
 	  if (def_argc)
-	    argcv_free (def_argc, def_argv);
-	  def_argc = argc;
-	  def_argv = argv;
-	  p_argv = argv + 1;
+	    argv_free (def_argv);
+	  wordsplit_get_words (&ws, &def_argc, &def_argv);
+	  p_argv = def_argv + 1;
 	}
       else
 	{
 	  VDETAIL (1, (_("Ignoring unrecognized line %d\n"), line));
-	  argcv_free (argc, argv);
 	}
     }
   fclose (fp);
   free (buf);
 
+  if (wsflags & WRDSF_REUSE)
+    wordsplit_free (&ws);
+
   if (!p_argv)
     VDETAIL (1, (_("No matching line found\n")));
   else
@@ -494,7 +492,7 @@ parse_netrc (const char *filename)
 	    assign_string (&auth_args.password, p_argv[1]);
 	  p_argv += 2;
 	}
-      argcv_free (def_argc, def_argv);
+      argv_free (def_argv);
     }
 }
 
@@ -506,7 +504,7 @@ get_input (const char *prompt)
 
   printf ("%s", prompt);
   fflush (stdout);
-  getline (&buf, &n, stdin);
+  xgetline (&buf, &n, stdin);
 
   n = strlen (buf);
   if (n > 1 && buf[n - 1] == '\n')
@@ -523,10 +521,13 @@ callback (Gsasl *ctx, Gsasl_session *sctx, Gsasl_property prop)
     {
     case GSASL_PASSWORD:
       if (auth_args.password == NULL)
-	auth_args.password = getpass (_("Password: "));
-
-      if (auth_args.password == NULL)
-	return GSASL_AUTHENTICATION_ERROR;
+	{
+	  if (anubis_getpass (_("Password: "), &auth_args.password))
+	    {
+	      error ("anubis_getpass: %s", strerror (errno));
+	      return GSASL_AUTHENTICATION_ERROR;
+	    }
+	}
       gsasl_property_set (sctx, prop, auth_args.password);
       break;
 
@@ -580,9 +581,13 @@ callback (Gsasl *ctx, Gsasl_session *sctx, Gsasl_property prop)
 
     case GSASL_PASSCODE:
       if (auth_args.passcode == NULL)
-	auth_args.passcode = getpass (_("Passcode: "));
-      if (auth_args.passcode == NULL)
-	return GSASL_AUTHENTICATION_ERROR;
+	{
+	  if (anubis_getpass (_("Passcode: "), &auth_args.passcode))
+	    {
+	      error ("anubis_getpass: %s", strerror (errno));
+	      return GSASL_AUTHENTICATION_ERROR;
+	    }
+	}
       gsasl_property_set (sctx, prop, auth_args.passcode);
       break;
 
@@ -771,9 +776,9 @@ int
 diff (char *file, ANUBIS_SMTP_REPLY repl)
 {
   const char *input = smtp_reply_line (repl, 0) + 4;
-  unsigned char sample[MD5_DIGEST_BYTES];
-  unsigned char digest[MD5_DIGEST_BYTES];
-  int len;
+  unsigned char *digest;
+  char const *err;
+  int rc;
   int fd;
 
   fd = open (file, O_RDONLY);
@@ -782,19 +787,18 @@ diff (char *file, ANUBIS_SMTP_REPLY repl)
       error (_("Cannot open file %s: %s"), file, strerror (errno));
       return CMP_ERROR;
     }
-  anubis_md5_file (digest, fd);
+  rc = anubis_md5_file (fd, &digest, &err);
   close (fd);
 
-  len = strlen (input);
-  if (len != sizeof digest * 2)
+  if (rc)
     {
-      error (_("Invalid MD5 digest: %s"), input);
+      error (_("Can't compute file digest: %s"), err);
       return CMP_ERROR;
     }
-  string_hex_to_bin (sample, (unsigned char*)input, len);
-
-  return memcmp (digest, sample, sizeof digest) == 0 ?
-                 CMP_UNCHANGED : CMP_CHANGED;
+  
+  rc = strcasecmp ((char*)digest, input) == 0 ? CMP_UNCHANGED : CMP_CHANGED;
+  free (digest);
+  return rc;
 }
 
 void
@@ -826,7 +830,7 @@ smtp_upload (char *rcname)
       return;
     }
 
-  while (getline (&buf, &n, fp) > 0 && n > 0)
+  while (xgetline (&buf, &n, fp) > 0 && n > 0)
     {
       size_t len = strlen (buf);
       if (len && buf[len - 1] == '\n')
@@ -856,10 +860,7 @@ synch (void)
   ANUBIS_SMTP_REPLY repl;
   char *rcname;
 
-  obstack_init (&input_stk);
-#ifdef USE_GNUTLS
   init_ssl_libs ();
-#endif
 
   VDETAIL (1, (_("Using remote SMTP %s:%d\n"), smtp_host, smtp_port));
   if (parse_host (smtp_host, smtp_port, &addr))
@@ -894,13 +895,11 @@ synch (void)
 
   smtp_ehlo (1);
 
-#ifdef USE_GNUTLS
   if (enable_tls && smtp_reply_has_capa (smtp_capa, "STARTTLS", NULL))
     {
       starttls ();
       smtp_ehlo (0);
     }
-#endif
 
   smtp_auth ();
   /* Get the capabilities */
@@ -913,7 +912,6 @@ synch (void)
       smtp_quit ();
       return 1;
     }
-
   
   send_line ("XDATABASE EXAMINE");
   smtp_get_reply (repl);
@@ -970,7 +968,7 @@ read_netrc (void)
 }
 
 void
-xalloc_die ()
+xnomem (void)
 {
   error ("%s", _("Not enough memory"));
   exit (1);
