@@ -2,7 +2,7 @@
    net.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001-2014 The Anubis Team.
+   Copyright (C) 2001-2024 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -186,7 +186,7 @@ bind_and_listen (char *host, unsigned int port)
   int sd = 0;
   unsigned long inaddr;
   struct sockaddr_in addr;
-  int true = 1;
+  int t = 1;
 
   memset (&addr, 0, sizeof (addr));
   addr.sin_family = AF_INET;
@@ -223,7 +223,7 @@ bind_and_listen (char *host, unsigned int port)
   else
     addr.sin_addr.s_addr = htonl (INADDR_ANY);
 
-  setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, &true, sizeof (true));
+  setsockopt (sd, SOL_SOCKET, SO_REUSEADDR, &t, sizeof (t));
 
   if (bind (sd, (struct sockaddr *) &addr, sizeof (addr)))
     anubis_error (EXIT_FAILURE, errno, _("bind() failed"));
@@ -341,7 +341,7 @@ void
 net_create_stream (NET_STREAM * str, int fd)
 {
   stream_create (str);
-  stream_set_io (*str, (void *) fd, NULL, NULL, NULL, NULL, NULL);
+  stream_set_io (*str, (void *) (ptrdiff_t) fd, NULL, NULL, NULL, NULL, NULL);
 }
 
 
