@@ -2,7 +2,7 @@
    tls.c
 
    This file is part of GNU Anubis.
-   Copyright (C) 2001-2014 The Anubis Team.
+   Copyright (C) 2001-2024 The Anubis Team.
 
    GNU Anubis is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
@@ -288,7 +288,8 @@ start_ssl_server (NET_STREAM sd_client, int verbose)
 static void
 verify_certificate (gnutls_session_t session)
 {
-  int status, rc;
+  int rc;
+  unsigned status;
 
   rc = gnutls_certificate_verify_peers2 (session, &status);
   if (rc)
@@ -340,14 +341,12 @@ verify_certificate (gnutls_session_t session)
 static int
 cipher_info (gnutls_session_t session)
 {
-  const char *tmp;
   gnutls_credentials_type_t cred;
   gnutls_kx_algorithm_t kx;
   int bits;
 
   kx = gnutls_kx_get (session);
-  tmp = gnutls_kx_get_name (kx);
-  fprintf (stderr, "- Key Exchange: %s\n", tmp);
+  fprintf (stderr, "- Key Exchange: %s\n", gnutls_kx_get_name (kx));
 
   cred = gnutls_auth_get_type (session);
   switch (cred)
@@ -375,21 +374,14 @@ cipher_info (gnutls_session_t session)
       break;
     }
 
-  tmp = gnutls_protocol_get_name (gnutls_protocol_get_version (session));
-  fprintf (stderr, _("- Protocol: %s\n"), tmp);
-
-  tmp =
-    gnutls_certificate_type_get_name (gnutls_certificate_type_get (session));
-  fprintf (stderr, _("- Certificate Type: %s\n"), tmp);
-
-  tmp = gnutls_compression_get_name (gnutls_compression_get (session));
-  fprintf (stderr, _("- Compression: %s\n"), tmp);
-
-  tmp = gnutls_cipher_get_name (gnutls_cipher_get (session));
-  fprintf (stderr, _("- Cipher: %s\n"), tmp);
-
-  tmp = gnutls_mac_get_name (gnutls_mac_get (session));
-  fprintf (stderr, _("- MAC: %s\n"), tmp);
+  fprintf (stderr, _("- Protocol: %s\n"),
+	   gnutls_protocol_get_name (gnutls_protocol_get_version (session)));
+  fprintf (stderr, _("- Certificate Type: %s\n"),
+	   gnutls_certificate_type_get_name (gnutls_certificate_type_get (session)));
+  fprintf (stderr, _("- Cipher: %s\n"),
+	   gnutls_cipher_get_name (gnutls_cipher_get (session)));
+  fprintf (stderr, _("- MAC: %s\n"),
+	   gnutls_mac_get_name (gnutls_mac_get (session)));
   return 0;
 }
 
